@@ -14,16 +14,15 @@ fn main() {
 
     // Define a basic route
     server.route(Method::GET, "/", |_req| {
-        Response::new(
-            200,
-            "Hello World!",
-            vec![Header::new("Content-Type", "text/plain")],
-        )
+        Response::new()
+            .status(200)
+            .text("Hello World!")
+            .header(Header::new("Content-Type", "text/plain"))
     });
 
     // Now to add some middleware
     // For this example we will add a middleware that will print all requests to the console
-    server.every(Box::new(|req| {
+    server.middleware(Box::new(|req| {
         println!("[{}] {} {}", req.address, req.method, req.path);
 
         // Return None to forward the request to the next middleware or route
