@@ -8,7 +8,10 @@ It can be disabled with the `cookies` feature.
 
 use std::fmt;
 
+use crate::common::decode_url;
+
 /// Represents a Cookie
+#[derive(Hash, PartialEq, Eq)]
 pub struct Cookie {
     /// Cookie Key
     pub name: String,
@@ -47,7 +50,11 @@ impl Cookie {
     /// use afire::Cookie;
     /// let cookie = Cookie::new("name", "value");
     /// ```
-    pub fn new(name: &str, value: &str) -> Cookie {
+    pub fn new<T, M>(name: T, value: M) -> Cookie
+    where
+        T: std::fmt::Display,
+        M: std::fmt::Display,
+    {
         Cookie {
             name: name.to_string(),
             value: value.to_string(),
@@ -74,7 +81,10 @@ impl Cookie {
                 }
                 .trim_end_matches(';');
 
-                final_cookies.push(Cookie::new(name, value));
+                final_cookies.push(Cookie::new(
+                    decode_url(name.to_owned()),
+                    decode_url(value.to_owned()),
+                ));
             }
             return Some(final_cookies);
         }
@@ -116,7 +126,11 @@ impl SetCookie {
     /// use afire::SetCookie;
     /// let cookie = SetCookie::new("name", "value");
     /// ```
-    pub fn new(name: &str, value: &str) -> SetCookie {
+    pub fn new<T, M>(name: T, value: M) -> SetCookie
+    where
+        T: std::fmt::Display,
+        M: std::fmt::Display,
+    {
         SetCookie {
             cookie: Cookie::new(name, value),
             max_age: None,
@@ -155,11 +169,11 @@ impl SetCookie {
     /// ```
     /// use afire::SetCookie;
     /// let mut cookie = SetCookie::new("name", "value")
-    ///     .set_max_age(10 * 60);
+    ///     .max_age(10 * 60);
     ///
     /// assert_eq!(cookie.max_age, Some(10*60));
     /// ```
-    pub fn set_max_age(self, max_age: u64) -> SetCookie {
+    pub fn max_age(self, max_age: u64) -> SetCookie {
         let mut new = self;
         new.max_age = Some(max_age);
         new
@@ -170,11 +184,14 @@ impl SetCookie {
     /// ```
     /// use afire::SetCookie;
     /// let mut cookie = SetCookie::new("name", "value")
-    ///     .set_domain("domain");
+    ///     .domain("domain");
     ///
     /// assert_eq!(cookie.domain, Some("domain".to_string()));
     /// ```
-    pub fn set_domain(self, domain: &str) -> SetCookie {
+    pub fn domain<T>(self, domain: T) -> SetCookie
+    where
+        T: fmt::Display,
+    {
         let mut new = self;
         new.domain = Some(domain.to_string());
         new
@@ -185,11 +202,14 @@ impl SetCookie {
     /// ```
     /// use afire::SetCookie;
     /// let mut cookie = SetCookie::new("name", "value")
-    ///     .set_path("path");
+    ///     .path("path");
     ///
     /// assert_eq!(cookie.path, Some("path".to_string()));
     /// ```
-    pub fn set_path(self, path: &str) -> SetCookie {
+    pub fn path<T>(self, path: T) -> SetCookie
+    where
+        T: fmt::Display,
+    {
         let mut new = self;
         new.path = Some(path.to_string());
         new
@@ -200,11 +220,11 @@ impl SetCookie {
     /// ```
     /// use afire::SetCookie;
     /// let mut cookie = SetCookie::new("name", "value")
-    ///     .set_secure(true);
+    ///     .secure(true);
     ///
     /// assert_eq!(cookie.secure, true);
     /// ```
-    pub fn set_secure(self, secure: bool) -> SetCookie {
+    pub fn secure(self, secure: bool) -> SetCookie {
         let mut new = self;
         new.secure = secure;
         new
