@@ -1,6 +1,7 @@
 use std::fmt;
 
 /// Methods for a request
+#[derive(Hash, PartialEq, Eq)]
 pub enum Method {
     /// GET Method
     ///
@@ -34,7 +35,7 @@ pub enum Method {
 
     /// PATCH Method
     ///
-    /// Used for aplaying a partial update to a resource
+    /// Used for applying a partial update to a resource
     PATCH,
 
     /// TRACE Method
@@ -69,7 +70,10 @@ impl Method {
     /// assert!(Method::from_string("TRACE") == Method::TRACE);
     /// assert!(Method::from_string("foo") == Method::CUSTOM("FOO".to_string()));
     /// ```
-    pub fn from_string(s: &str) -> Method {
+    pub fn from_string<T>(s: T) -> Method
+    where
+        T: fmt::Display,
+    {
         let upper_s = s.to_string().to_uppercase();
         match &upper_s[..] {
             "GET" => Method::GET,
@@ -135,14 +139,3 @@ impl fmt::Debug for Method {
             .finish()
     }
 }
-
-impl PartialEq for Method {
-    /// Allow comparing Method Enums
-    ///
-    /// EX: Method::GET == Method::GET
-    ///
-    /// > True
-    fn eq(&self, other: &Self) -> bool {
-        std::mem::discriminant(self) == std::mem::discriminant(other)
-    }
-}
